/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : PathBuilder.h                                            */
/*                                                                       */
/*       Date : September 20, 1998                                       */
/* Updated at : December  28, 1998                                       */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/
/**
    Cycle starts from and ends at the same base city.
    If the base city constraint is on, then cycles they do not start and
    end at the same base city will be placed in paths.
    If the chain builds a path, then it will be placed in paths.
**/
/*************************************************************************/

#ifndef _PATHBUILDER_H
#define _PATHBUILDER_H

#include <WeightedRandomPickerSet.h>

#include "ListOfEdgeLists.h"
#include "EdgeGraph.h"
#include "Flight.h"
#include "AirScheduleProblem.h"

#include <iostream.h>

class PathBuilder {

  public :

     PathBuilder( AirScheduleProblem *_myProblem ) { myProblem=_myProblem; };

     void buildPaths( EdgeGraph& eg, ListOfEdgeLists& cycles, 
		      ListOfEdgeLists& paths, int FlightNumber, 
		      float maxLength);

     void buildPathsBackwards( EdgeGraph& eg, ListOfEdgeLists& cycles, 
		               ListOfEdgeLists& paths, int FlightNumber,
			       float maxLength);

     void buildCycles( EdgeGraph& eg, ListOfEdgeLists& cycles,
                       int FlightNumber, float maxLength, int APPLYBASECITY=1);

     void buildCyclesN( EdgeGraph& eg, ListOfEdgeLists& cycles,
                        int FlightNumber, float maxLength, int numOfSolutions, 
		        int APPLYBASECITY=1);

  private :
    AirScheduleProblem *myProblem;

    lookAheadForCycles( EdgeGraph& eg, Edge** cstack, int& deep, 
			int maxdeep, Flight *fflight, float maxLength);

    void searchForwardWithEdgeSet( EdgeGraph& eg, 
				   WeightedRandomPickerSet<Edge*>& candids,
 				   ListOfEdgeLists& retCycles, 
 				   ListOfEdgeLists& paths, float maxLength,
				   int APPLYBASECITY=1);

    lookBackForCycles( EdgeGraph& eg, Edge** cstack, int deep, 
		       int maxdeep, Flight *fflight, float maxLength);

    void searchBackwardWithEdgeSet( EdgeGraph& eg, 
				    WeightedRandomPickerSet<Edge*>& candids,
 				    ListOfEdgeLists& retCycles, 
 				    ListOfEdgeLists& paths, float maxLength,
                                    int APPLYBASECITY=1);

    void println(WeightedRandomPickerSet<Edge*>* _set);

    searchForCycles( EdgeGraph& eg, Edge** cstack, ListOfEdgeLists& retCycles, 
		     int& first, int middle, int& last, float maxLength, 
		     int APPLYBASECITY=1);

    void searchBothWithEdgeSet( EdgeGraph& eg, 
				WeightedRandomPickerSet<Edge*>& candids,
                                ListOfEdgeLists& retCycles, float maxLength,
				int numOfSolutions, int APPLYBASECITY=1);



}; // end of Class PathBuilder

#endif 
